﻿using DotNetNote.Models;
using Microsoft.AspNet.Hosting;
using Microsoft.AspNet.Http;
using Microsoft.AspNet.Mvc;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Http.Headers;

namespace DotNetNote.Controllers
{
    public class DotNetNoteController : Controller
    {
        private IHostingEnvironment _environment; // 웹 사이트 루트 폴더 구하기: wwwroot
        private INoteRepository _repository;
        private INoteCommentRepository _commentRepository;
        private readonly IConfiguration _config;

        // 공통 속성: 검색 모드: 검색 모드이면 true, 그렇지 않으면 false.
        public bool SearchMode { get; set; } = false;
        public string SearchField { get; set; }
        public string SearchQuery { get; set; }

        public int intPage = 0; // 현재 보여줄 페이지 번호
        public int intTotalCount = 0;//총 레코드 갯수(글번호 순서 정렬용)

        public DotNetNoteController(IHostingEnvironment environment, INoteRepository repository, INoteCommentRepository commentRepository, IConfiguration config)
        {
            _environment = environment;
            _repository = repository;
            _commentRepository = commentRepository;
            _config = config;
        }

        public IActionResult Index()
        {
            // 검색 모드 결정
            SearchMode = (!string.IsNullOrEmpty(Request.Query["SearchField"]) && !string.IsNullOrEmpty(Request.Query["SearchQuery"]));
            if (SearchMode)
            {
                SearchField = Request.Query["SearchField"].ToString().Replace("--", "");
                SearchQuery = Request.Query["SearchQuery"].ToString().Replace("--", ""); // 꼭 보안 함수로 묶을 것...
            }
            
            //[1] 쿼리스트링에 따른 페이지 보여주기
            if (!string.IsNullOrEmpty(Request.Query["Page"].ToString()))
            {
                intPage = Convert.ToInt32(Request.Query["Page"]) - 1; // Page는 보여지는 쪽은 1, 2, 3, ... 코드단에서는 0, 1, 2, ...
            }
            else
            {
                intPage = 0;
            }

            //[2] 쿠키를 사용한 리스트 페이지 번호 유지 적용: 100번째 페이지 보고 있다고 다시 리스트 왔을 때 100번째 페이지 표시
            if (!string.IsNullOrEmpty(Request.Cookies["DotNetNote_PageNum"]))
            {
                if (!String.IsNullOrEmpty(Request.Cookies["DotNetNote_PageNum"]))
                {
                    intPage = Convert.ToInt32(Request.Cookies["DotNetNote_PageNum"]);
                }
                else
                {
                    intPage = 0;
                }
            }

            IEnumerable<Note> r;  
            if (!SearchMode)
            {
                intTotalCount = _repository.GetCount();
                r = _repository.GetAll(intPage);
            }
            else
            {
                intTotalCount = _repository.GetCountBySearch(SearchField, SearchQuery);
                r = _repository.GetSeachAll(intPage, SearchField, SearchQuery);
            }

            ViewBag.TotalRecord = intTotalCount;
            ViewBag.SearchMode = SearchMode;
            ViewBag.SearchField = SearchField;
            ViewBag.SearchQuery = SearchQuery;

            return View(r);
        }

        public IActionResult Create()
        {
            ViewBag.FormType = DotNetNote.BoardWriteFormType.Write;
            ViewBag.TitleDescription = "글 쓰기 - 다음 필드들을 채워주세요.";
            ViewBag.SaveButtonText = "저장"; 


            return View();
        }

        [HttpPost]
        public IActionResult Create(Note model, ICollection<IFormFile> files)
        {
            //파일 업로드 처리 시작
            string fileName = String.Empty;
            int fileSize = 0;

            var uploadFolder = Path.Combine(_environment.WebRootPath, "files");

            foreach (var file in files)
            {
                if (file.Length > 0)
                {
                    if (Convert.ToBoolean(_config.GetSection("Storage").GetSection("AZURE_STORAGE_ENABLE").Value))
                    {
                        // Azure Storage 업로드 코드 들어오는 곳...

                    }
                    else
                    {
                        // 로컬 업로드

                        fileSize = Convert.ToInt32(file.Length);
                        // 파일명 중복 처리
                        fileName = Dul.FileUtility.GetFileNameWithNumbering(uploadFolder, Path.GetFileName(ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"')));
                        //fileName = Path.GetFileName(ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"'));
                        file.SaveAs(Path.Combine(uploadFolder, fileName)); // 업로드

                    }
                }
            }
            
            Note note = new Note();

            //note.Id = -1; // 입력 페이지는 일련번호로 설정됨 

            note.Name = model.Name;
            note.Email = Dul.HtmlUtility.Encode(model.Email);
            note.Homepage = model.Homepage;
            note.Title = Dul.HtmlUtility.Encode(model.Title);
            note.Content = model.Content;
            note.FileName = fileName;
            note.FileSize = fileSize;
            note.Password = model.Password;

            if (HttpContext.Connection.RemoteIpAddress != null)
            {
                note.PostIp = HttpContext.Connection.RemoteIpAddress.ToString(); // IP 주소
            }
            else
            {
                note.PostIp = "";
            }

            note.Encoding = model.Encoding;

            //NoteRepository repository = new NoteRepository();
            _repository.Add(note); // 데이터 저장

            TempData["Message"] = "데이터가 저장되었습니다.";

            return RedirectToAction("Index"); // 리스트 페이지로 이동
        }


        /// <summary>
        /// 게시판 파일 강제 다운로드 기능(BoardDown, BoardDown.aspx)
        /// </summary>
        public FileResult BoardDown(int id)
        {
            string fileName = "";


            // 넘겨져 온 번호에 해당하는 파일명 가져오기(보안때문에... 파일명 숨김)
            fileName = _repository.GetFileNameById(id);

            if (fileName == null)
            {
                return null;
            }
            else
            {
                // 다운로드 카운트 증가 메서드 호출
                _repository.UpdateDownCount(fileName);
                byte[] fileBytes = System.IO.File.ReadAllBytes(
                    Path.Combine(_environment.WebRootPath, "files") + "\\" + fileName);

                //return File(fileBytes, System.Net.Mime.MediaTypeNames.Application.Octet, fileName);
                return File(fileBytes, "application/octet-stream", fileName);
            }
        }


        /// <summary>
        /// 게시판의 상세 보기 페이지(Details, BoardView)
        /// </summary>
        public IActionResult Details(int id)
        {
            // 넘겨온 Id 값에 해당하는 레코드 하나 읽어서 Note 클래스에 바인딩
            var note = _repository.GetNoteById(id);

            string strContent = note.Content;

            // 인코딩 방식에 따른 데이터 출력
            string strEncoding = note.Encoding;
            string rEncoding = ""; 
            if (strEncoding == "Text") // Text : 소스 그대로 표현
            {
                rEncoding = strContent.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;").Replace("\r\n", "<br />");
            }
            else if (strEncoding == "Mixed") // Mixed : 엔터처리만
            {
                rEncoding = strContent.Replace("\r\n", "<br />");
            }
            else // HTML : HTML 형식으로 출력
            {
                rEncoding = strContent; // 변환없음
            }
            ViewBag.Content = rEncoding;


            if (note.FileName.Length > 1)
            {
                ViewBag.FileName = String.Format("<a href='/DotNetNote/BoardDown?Id={0}'>" + "{1}{2} / 전송수: {3}</a>", note.Id, "<img src=\"/images/ext/ext_zip.gif\" border=\"0\">", note.FileName, note.DownCount);
                if (Dul.BoardLibrary.IsPhoto(note.FileName))
                {
                    ViewBag.ImageDown = $"<img src=\'/DotNetNote/ImageDown?Id={note.Id}\'><br />";
                }
            }
            else
            {
                ViewBag.FileName = "(업로드된 파일이 없습니다.)";
            }


            // 현재 글에 해당하는 댓글 리스트와 현재 글 번호를 담아서 전달
            NoteCommentViewModel vm = new NoteCommentViewModel();
            vm.NoteCommentList = _commentRepository.GetNoteComments(note.Id);
            vm.BoardId = note.Id;
            ViewBag.CommentListAndId = vm;  
            
            return View(note); 
        }


        [HttpGet]
        public IActionResult Delete(int id)
        {
            ViewBag.Id = id; 
            return View();
        }


        [HttpPost]
        public IActionResult Delete(int id, string Password)
        {
            if (_repository.DeleteNote(id, Password) > 0)
            {
                TempData["Message"] = "데이터가 삭제되었습니다.";
                //return RedirectToAction("DeleteCompleted");
                return RedirectToAction("Index");
            }
            else
            {
                ViewBag.Message = "삭제되지 않았습니다. 비밀번호를 확인하세요.";
            }

            ViewBag.Id = id;
            return View();
        }

        public IActionResult DeleteCompleted()
        {
            return View();
        }


        [HttpGet]
        public IActionResult Edit(int id)
        {
            ViewBag.FormType = DotNetNote.BoardWriteFormType.Modify;
            ViewBag.TitleDescription = "글 수정 - 아래 항목을 수정하세요.";
            ViewBag.SaveButtonText = "수정";

            // 기존 데이터를 바인딩
            var note = _repository.GetNoteById(id);

            // 첨부된 파일명 및 파일크기 기록
            if (note.FileName.Length > 1)
            {
                ViewBag.FileName = note.FileName;
                ViewBag.FileSize = note.FileSize;
                ViewBag.FileNamePrevious = $"기존에 업로드된 파일명: {note.FileName}";
            }
            else
            {
                ViewBag.FileName = "";
                ViewBag.FileSize = 0;
            }

            return View(note);
        }

        [HttpPost]
        public IActionResult Edit(Note model, ICollection<IFormFile> files, int id, string previousFileName = "", int previousFileSize = 0)
        {
            string fileName = "";
            int fileSize = 0;

            if (previousFileName != null)
            {
                fileName = previousFileName;
                fileSize = previousFileSize;
            }

            //파일 업로드 처리 시작
            var uploadFolder = Path.Combine(_environment.WebRootPath, "files");

            foreach (var file in files)
            {
                if (file.Length > 0)
                {
                    if (Convert.ToBoolean(_config.GetSection("Storage").GetSection("AZURE_STORAGE_ENABLE").Value))
                    {
                        // Azure Storage 업로드 코드 들어오는 곳...

                    }
                    else
                    {
                        // 로컬 업로드
                        fileSize = Convert.ToInt32(file.Length);
                        // 파일명 중복 처리
                        fileName = Dul.FileUtility.GetFileNameWithNumbering(uploadFolder, Path.GetFileName(ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"')));
                        file.SaveAs(Path.Combine(uploadFolder, fileName)); // 업로드
                    }
                }
            }


            Note note = new Note();

            note.Id = id;

            note.Name = model.Name;
            note.Email = Dul.HtmlUtility.Encode(model.Email);
            note.Homepage = model.Homepage;
            note.Title = Dul.HtmlUtility.Encode(model.Title);
            note.Content = model.Content;
            note.FileName = fileName;
            note.FileSize = fileSize;
            note.Password = model.Password;
            if (HttpContext.Connection.RemoteIpAddress != null)
            {
                note.ModifyIp = HttpContext.Connection.RemoteIpAddress.ToString(); // IP 주소
            }
            else
            {
                note.ModifyIp = "";
            }

            note.Encoding = model.Encoding;

            int r = _repository.UpdateNote(note);
            if (r > 0)
            {
                TempData["Message"] = "수정되었습니다.";
                return RedirectToAction("Details", new { Id = id });
            }
            else
            {
                ViewBag.ErrorMessage = "업데이트가 되지 않았습니다. 암호를 확인하세요.";
                return View(note);
            }
        }

        
        /// <summary>
        /// 답변 글쓰기 폼
        /// </summary>
        /// <param name="id">부모글 Id</param>
        [HttpGet]
        public IActionResult Reply(int id)
        {
            ViewBag.FormType = DotNetNote.BoardWriteFormType.Reply;
            ViewBag.TitleDescription = "글 답변 - 다음 필드들을 채워주세요.";
            ViewBag.SaveButtonText = "답변";

            // 기존 데이터를 바인딩
            var note = _repository.GetNoteById(id); // 기존 부모글 Id

            // 새로운 Note 개체 생성
            var newNote = new Note(); 

            // 기존 글의 제목과 내용을 새 Note 개체에 저장 후 전달
            newNote.Title = $"Re : {note.Title}";
            newNote.Content = $"\n\nOn {note.PostDate}, '{note.Name}' wrote:\n----------\n>{note.Content.Replace("\n", "\n>")}\n---------";
            
            return View(newNote);
        }

        /// <summary>
        /// 답변 글쓰기 처리
        /// </summary>
        [HttpPost]
        public IActionResult Reply(Note model, ICollection<IFormFile> files, int id)
        {
            //파일 업로드 처리 시작
            string fileName = String.Empty;
            int fileSize = 0;

            var uploadFolder = Path.Combine(_environment.WebRootPath, "files");

            foreach (var file in files)
            {
                if (file.Length > 0)
                {

                    if (Convert.ToBoolean(_config.GetSection("Storage").GetSection("AZURE_STORAGE_ENABLE").Value))
                    {
                        // Azure Storage 업로드 코드 들어오는 곳...

                    }
                    else
                    {
                        // 로컬 업로드
                        fileSize = Convert.ToInt32(file.Length);
                        fileName = Dul.FileUtility.GetFileNameWithNumbering(uploadFolder, Path.GetFileName(ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"')));
                        file.SaveAs(Path.Combine(uploadFolder, fileName));
                    }

                }
            }
                
            Note note = new Note();

            note.Id = note.ParentNum = Convert.ToInt32(id); // 부모글 저장

            note.Name = model.Name;
            note.Email = Dul.HtmlUtility.Encode(model.Email);
            note.Homepage = model.Homepage;
            note.Title = Dul.HtmlUtility.Encode(model.Title);
            note.Content = model.Content;
            note.FileName = fileName;
            note.FileSize = fileSize;
            note.Password = model.Password;
            if (HttpContext.Connection.RemoteIpAddress != null)
            {
                note.PostIp = HttpContext.Connection.RemoteIpAddress.ToString(); // IP 주소
            }
            else
            {
                note.PostIp = "";
            }
            note.Encoding = model.Encoding;

            _repository.ReplyNote(note); // 데이터 답변 저장

            TempData["Message"] = "데이터가 저장되었습니다.";

            return RedirectToAction("Index");
        }


        /// <summary>
        /// ImageDown : 완성형(DotNetNote) 게시판의 이미지전용다운 페이지
        /// 이미지 경로를 보여주지 않고 다운로드: 대용량 사이트 운영시 직접 이미지 경로 사용 권장(CDN 사용)
        /// /DotNetNote/ImageDown/1234 => 1234번 이미지 파일을 강제 다운로드
        /// <img src="/DotNetNote/ImageDown/1234" /> => 이미지 태그 실행
        /// </summary>
        public FileResult ImageDown(int id)
        {
            string fileName = "";

            // 넘겨져 온 번호에 해당하는 파일명 가져오기(보안때문에... 파일명 숨김)
            fileName = _repository.GetFileNameById(id);

            if (fileName == null)
            {
                return null;
            }
            else
            {
                string strFileName = fileName;
                string strFileExt = Path.GetExtension(strFileName);
                string strContentType = "";
                if (strFileExt == ".gif" || strFileExt == ".jpg" || strFileExt == ".jpeg" || strFileExt == ".png")
                {
                    switch (strFileExt)
                    {
                        case ".gif":
                            strContentType = "image/gif"; break;
                        case ".jpg":
                            strContentType = "image/jpeg"; break;
                        case ".jpeg":
                            strContentType = "image/jpeg"; break;
                        case ".png":
                            strContentType = "image/png"; break;
                    }
                }

                // 다운로드 카운트 증가 메서드 호출
                _repository.UpdateDownCount(fileName);
                byte[] fileBytes = System.IO.File.ReadAllBytes(Path.Combine(_environment.WebRootPath, "files") + "\\" + fileName);

                return File(fileBytes, strContentType, fileName);
            }
        }


        [HttpPost]
        public IActionResult CommentAdd(int BoardId, string txtName, string txtPassword, string txtOpinion)
        {
            NoteComment comment = new NoteComment();
            comment.BoardId = BoardId;
            comment.Name = txtName;
            comment.Password = txtPassword;
            comment.Opinion = txtOpinion;


            _commentRepository.AddNoteComment(comment);


            return RedirectToAction("Details", new { Id = BoardId }); 
        }


        [HttpGet]
        public IActionResult CommentDelete(string BoardId, string Id)
        {
            ViewBag.BoardId = BoardId;
            ViewBag.Id = Id;

            return View();
        }


        [HttpPost]
        public IActionResult CommentDelete(string BoardId, string Id, string txtPassword)
        {
            if (_commentRepository.GetCountBy(Convert.ToInt32(BoardId), Convert.ToInt32(Id), txtPassword) > 0)
            {
                _commentRepository.DeleteNoteComment(Convert.ToInt32(BoardId), Convert.ToInt32(Id), txtPassword);
                return RedirectToAction("Details", new { Id = BoardId });
            }

            ViewBag.BoardId = BoardId;
            ViewBag.Id = Id;
            ViewBag.ErrorMessage = "암호가 틀립니다. 다시 입력해주세요.";

            return View();
        }
        
    }
}
